+++
fragment = "content"
date = "2019-10-24"
weight = 100

title = "Guidelines Gitlab-based packaging workflow"
+++

# GitLab-based packaging workflow

Packaging in Apertis is handled through a set of git repositories hosted
on GitLab containing the sources for every packages. GitLab CI pipelines
take care of propagating updates to OBS to build and publish the
resulting binaries.

## Repository contents

The packaging git repositories follow the
[DEP-14](http://dep.debian.net/deps/dep14/) Git layout specification
with the following conventions:

  - `upstream/latest` branches with the unpacked upstream project code
    from the debian package)
      - FIXME: change to: `upstream/${UPSTREAM_DISTRIBUTION}` branches
        with the unpacked upstream project code from the debian package
        (e.g. `upstream/buster`)
  - `debian/${UPSTREAM_DISTRIBUTION}` branches with the Debian changes
    on top of the upstream project code (e.g. `debian/buster`)
  - `debian/${UPSTREAM_VERSION}` tags (e.g. `apertis/2.48.0-1`)
  - `apertis/${APERTIS_RELEASE}` branches where the debian branches
    above get merged with the Apertis packaging (e.g.
    `apertis/v2019pre`)
  - `apertis/${APERTIS_VERSION}` tags (e.g. `apertis/2.48.0-1co1`)
  - `pristine-lfs` branch containing the information to fetch the
    original tarballs stored on Git-LFS via
    [pristine-lfs](https://gitlab.apertis.org/infrastructure/pristine-lfs)

In addition, all the packaging repositories contains a
`debian/apertis/gitlab-ci.yml` file to enable the CI pipeline which
generates source packages and uploads them to OBS.

## Development environment

All the instructions below assume an Apertis development enviroment:
either boot the Apertis [SDK]( {{< ref "/sdk.md" >}} ) or enter the
`apertis-*-package-source-builder` Docker container: <code>

    APERTIS_RELEASE=v2019pre
    docker pull docker-registry.apertis.org/apertis/apertis-${APERTIS_RELEASE}-package-source-builder
    docker run -it --rm --env HOME -w "$(pwd)" -v "$HOME:$HOME" -v "$HOME:/root" --security-opt label=disable apertis-${APERTIS_RELEASE}-package-source-builder bash

</code>

## How to sync an Apertis package with a new version from Debian

1.  Check out the source repository from GitLab <code>
        UPSTREAM_DISTRIBUTION=buster
        UPSTREAM_VERSION=2.58.3-2
        UPSTREAM=http://deb.debian.org/debian/pool/main/g/glib2.0/glib2.0_${UPSTREAM_VERSION}.dsc
        PACKAGE=glib2.0
        COMPONENT=target

        git clone git@gitlab.apertis.org:pkg/${COMPONENT}/${PACKAGE}
        cd ${PACKAGE}
    </code>
2.  Sync the upstream contents <code>
        git branch debian/${UPSTREAM_DISTRIBUTION} origin

        GBP_CONF_FILES=/etc/git-buildpackage/gbp.conf gbp import-dsc ${UPSTREAM} --debian-branch=debian/${UPSTREAM_DISTRIBUTION} --upstream-branch=upstream/${UPSTREAM_DISTRIBUTION} --no-pristine-tar --allow-unauthenticated
    </code>
3.  Commit the orig tarball to pristine-lfs <code>
        #FIXME: We should consider using 'import-tarballs' script instead, which acts nicely on .dsc files

        pristine-lfs commit glib2.0_2.58.3.orig.tar.gz
    </code>
4.  Prepare the repository for the merge [by setting up
    `dpkg-mergechangelogs`](https://manpages.debian.org/testing/dpkg-dev/dpkg-mergechangelogs.1.en.html#INTEGRATION_WITH_GIT)
    first and then by configuring it in the repository <code>
        git config merge.dpkg-mergechangelogs.driver "dpkg-mergechangelogs -m %O %A %B %A"
        echo 'debian/changelog merge=dpkg-mergechangelogs' > .git/info/attributes
    </code>
5.  Merge the upstream changes <code>
        git checkout -b wip/$USER/upstream-sync

        git merge --no-commit debian/{UPSTREAM_DISTRIBUTION}

        dch -l co

        # in the editor add some contents to the created changelog entry, leaving it UNRELEASED, for instance:
        #
        # glib2.0 (2.58.3-2co1) UNRELEASED; urgency=medium
        #
        #  * Sync with Debian Buster
        #  * Remaining Apertis changes:
        #    - Disable all tests

        git add ./debian/changelog

        git commit --signoff -m "Sync with Debian ${UPSTREAM_DISTRIBUTION} ${PACKAGE} ${UPSTREAM_VERSION}"
    </code>
6.  Push your changes for review <code>
        git push origin debian/${UPSTREAM_DISTRIBUTION} debian/${UPSTREAM_VERSION}
        git push -o merge_request.create
    </code>

## How to work with downstream packaging changes

The packaging repositories follow the [Apertis contribution
process](https://designs.apertis.org/latest/contribution-process.html).

However, the deb packaging format does not allow changes outside of the
`debian/` folder: changes to the upstream files should be carried as
patches, shipped in `debian/patches` folder.

To make working with those patches a bit easier for those used to `git
rebase`, [the `gbp pq` tool provides a way to convert patches to git
branches and
back](https://honk.sigxcpu.org/projects/git-buildpackage/manual-html/gbp.patches.html).

## How to issue a release

Once all the other changes have been reviewed and landed following the
[Apertis contribution
process](https://designs.apertis.org/latest/contribution-process.html) a
package release need to be issues to land the update to the package pool
in OBS.

The only things needed to do so are updating the `debian/changelog` file
and pushing the tagged commit to the Apertis release branch:

1.  Finalize the changelog, commit and tag <code>
        GBP_CONF_FILES=/etc/git-buildpackage/gbp.conf gbp dch -R -D apertis --debian-branch=apertis/${APERTIS_RELEASE}

        git add ./debian/changelog

        git commit --signoff -m "Release $(dpkg-parsechangelog -S source) $(dpkg-parsechangelog -S version)"

        GBP_CONF_FILES=/etc/git-buildpackage/gbp.conf gbp buildpackage --git-tag --git-tag-only --git-debian-branch=apertis/${APERTIS_RELEASE} --git-debian-tag='apertis/%(version)s'
    </code>
2.  Push the release commit and tag <code>
        git push origin HEAD apertis/$(dpkg-parsechangelog -S version)
    </code>

## How to add a new packaging repository

1.  Initialize a git source repository and import the upstream tarball
    <code>
        UPSTREAM_DISTRIBUTION=buster
        UPSTREAM_TARBALL=https://github.com/xfce-mirror/thunarx-python/archive/thunarx-python-0.5.1.tar.gz
        PACKAGE=thunarx-python
        COMPONENT=target

        git init ${PACKAGE}
        cd ${PACKAGE}

        gbp import-orig ${UPSTREAM_TARBALL} --debian-branch=debian/${UPSTREAM_DISTRIBUTION} --upstream-branch=upstream/${UPSTREAM_DISTRIBUTION} --no-pristine-tar
    </code>
2.  Create pristine-lfs branch and commit upstream tarball <code>
        pristine-lfs commit ../thunarx-python_0.5.1.orig.tar.gz
    </code>
3.  Add generic debian/ packaging to debian/buster <code>
        cp -a ~/sample-debian-packaging-folder/ debian/
        git add debian/
        git commit -m "Add initial debian/ packaging" debian/
    </code>
4.  Add Apertis specific packaging changes <code>
        git checkout -b apertis/${APERTIS_RELEASE} debian/${UPSTREAM_DISTRIBUTION}

        # Add Apertis specific changes

        git commit -m "Apertis specific changes" debian/
    </code>
5.  Prepare changelog for release <code>
        dch -l co

        # in the editor add some contents to the created changelog entry, leaving it UNRELEASED, for instance:
        #
        # thunarx-python (0.5.1-1co1) UNRELEASED; urgency=medium
        #
        #  * New package import
        #  * Apertis specific changes:
        #    - Add patch to introduce an apertis specific package

        git add ./debian/changelog

        git commit --signoff -m "Sync with Debian ${UPSTREAM_DISTRIBUTION} ${PACKAGE} ${UPSTREAM_VERSION}"
    </code>
6.  Push your changes to gitlab <code>
        # Create a repository on the gitlab infra under proper group. Eg: git@gitlab.apertis.org:pkg/development/thunarx-python.git
        # And then add the same remote to your git repository to push your changes to the remote server.

        git remote add origin git@gitlab.apertis.org:pkg/development/thunarx-python.git
        git push origin --all
    </code>